/* AD9833.C

This program uses serial port B and PB6 for the AD9833

Note for the Ramp mode:
Since I want to reduce the processing time, I convert the start, stop and step
size frequencies to the values required by the DDS IC by multiplying each by
the conversion factor.  This causes the actual output frequencies to be in
error - always on the low side.  The error increases the further the output
frequency is from the start value.  The error will be less the closer the step
size multipled by the conversion factor (currently 10.73742 ) is to an integer.
For instance, a step size of 5000 will yield fairly accurate steps.  After about
10 steps the frequency will be off by 1 Hz.
*/

#define SERB_USEPORTD

//#define AD9833_DEBUG_PRINT 0x0F
//#define AD9833_DEBUG debug
//#define LCD_DEBUG debug

// define the LCD size for the HD44780 library
#define CMND_ADDR	0x2000			// external I/O address of the LCD
#define LCD_ROWS			2
#define LCD_COLUMNS		16

#use AD9833.lib
#use HD44780.lib

#define NbrMenuItems 7
const char MenuItems[NbrMenuItems][5] =
	{	"F0", "F1", "P0", "P1", "Wave", "Mode", "Ampl" };

const char DDSWAVES[5][4] =
	{ " RE", " SI", " TR", " SQ", " S2" };

#define NbrWaves 5
const char DDSWAVES_NAMES[NbrWaves][9] =
	{ "Reset   ", "Sine    ", "Triangle", "Square  ", "Square/2" };

#define NbrModes 5
const char DDSMODES[NbrModes][8] =
	{  "CW-F0  ", "CW-F1  ", "FSK/PSK", "R1amp  ", "R2amp  " };

long	StepSize, AdjCurFreq, SweepCount;// step size for ramp, ramp frequency
int	StepDuration, ExitFlag;				// delay increment in msec for ramp

void	DDSramp (void);
int	SetRampFreq ( void );

void TestPSK ( void );

main ()
{  int	MenuItemNbr, WaveForm;
	long	AdjIniFreq, AdjFinFreq, AdjStepSize;
   char	sAmpl[10];

	StepSize = 1000L;
   StepDuration = 0;
	brdInit();									// init CPU for prototype board
	LCDinit();									// init the display
   LCDprint ( "DDS by K3PTO" );
   LCDline (2);
   LCDprint ( "1Hz to 10MHz" );
   MsDelay ( 2000 );
   LCDclear ();
	AD9833init ();								// and the AD9833

// set initial frequency and display on LCD
   SetFreq ( 0, 20000, 0 ); 				// init F0/P0
   SetFreq ( 1, 100000, 180 );			// init F1/P1
   SetChannel ( OUT_F0|OUT_P0 );			// select channel
   SetWaveForm ( OUT_SINE );				// and waveform


	for ( ;; )									// forever loop
   {
		ShowDDS ();								// show DDS selections
// loop here until SW1 is pressed to enter menu system
		while ( !(Switches() & SW1) );	// wait for switch 1 press
		LCDclear();
      while ( SwitchVal & SW1 )			// stay in loop as long as SW1 pressed
      {
			for ( MenuItemNbr = 0;  MenuItemNbr < NbrMenuItems;  MenuItemNbr++ )
   	   {
				LCDclear();
				LCDprint ( "Change " );
				LCDprint ( MenuItems[MenuItemNbr] );
				MsDelay ( 1000 );
            if ( !(Switches() & SW1) ) break; // exit if SW1 released
			}
      } // wait for SW1 release

		switch (MenuItemNbr)
		{
      case 0:
			sprintf ( LCDstring, "F%d = %07ld", 0, DDSfreq[0] );
			DDSfreq[0] = LCDvalue(7);		// get and save new frequency
         SetFreq ( 0, DDSfreq[0], DDSphase[0] );
         break;

      case 1:
			sprintf ( LCDstring, "F%d = %07ld", 1, DDSfreq[1] );
			DDSfreq[1] = LCDvalue(7);		// get and save new frequency
         SetFreq ( 1, DDSfreq[1], DDSphase[1] );
         break;

      case 2:
			sprintf ( LCDstring, "P%d = %03d", 0, DDSphase[0] );
			DDSphase[0] = (int)LCDvalue(3); // get and save new phase
         SetFreq ( 0, DDSfreq[0], DDSphase[0] );
         break;

      case 3:
			sprintf ( LCDstring, "P%d = %03d", 1, DDSphase[1] );
			DDSphase[1] = (int)LCDvalue(3); // get and save new phase
         SetFreq ( 1, DDSfreq[1], DDSphase[1] );
         break;

      case 4:	// Wave
      	sprintf ( LCDstring, "\rWaveform menu" );
			LCDprint ( LCDstring );
			WaveForm = LCDmenu ( (char*)DDSWAVES_NAMES, NbrWaves );
         switch ( WaveForm )
         {
         case 0: SetWaveForm (OUT_OFF); break;
         case 1: SetWaveForm (OUT_SINE); break;
         case 2: SetWaveForm (OUT_TRIANGLE); break;
         case 3: SetWaveForm (OUT_SQUARE); break;
         case 4: SetWaveForm (OUT_SQUARE_DIV2); break;
         }
      	break;

      case 5:	// Mode
      	sprintf ( LCDstring, "\rMode menu" );
			LCDprint ( LCDstring );
			DDSmode = LCDmenu ( (char*)DDSMODES, NbrModes );
         switch ( DDSmode )
         {
         case 0: SetChannel (OUT_F0|OUT_P0); break;
         case 1: SetChannel (OUT_F1|OUT_P1); break;
         case 2:         					// FSK/PSK
				Set_SK();
            break;
         case 3:
         case 4:
            SetChannel (OUT_F0|OUT_P0); // start with F0 & P0
         	DDSramp();						// get the sweep step size and delay
				ShowDDS ();						// show DDS selections
				AdjIniFreq = (long) ( DDSfreq[0]*FREQ_FACTOR ); // adj for scaling
            AdjFinFreq = (long) ( DDSfreq[1]*FREQ_FACTOR );	// adj for scaling
				AdjStepSize = (long) ( StepSize*FREQ_FACTOR );	// adj for scaling
				SweepCount = 1;				// odd = start with sweep up
				ExitFlag = 0;
				while ( !ExitFlag )			// forever loop
            {
					if ( SweepCount & 1 )	// ramp up
	            {	for ( AdjCurFreq=AdjIniFreq; AdjCurFreq<=AdjFinFreq; AdjCurFreq+=AdjStepSize )
							if ( SetRampFreq() ) break;
               }
   				else							// ramp down
	            {	if (AdjCurFreq > AdjFinFreq ) AdjCurFreq -= AdjStepSize;
               	for ( AdjCurFreq-=AdjStepSize; AdjCurFreq>=AdjIniFreq+AdjStepSize; AdjCurFreq-=AdjStepSize )
							if ( SetRampFreq() ) break;
               }
               if ( DDSmode == 4 ) SweepCount+=1; // allow sweep down
            } // forever loop
				LCDclear ();					// show ramp terminated
				while ( Switches() & SW1 ); // wait for SW1 release
	         SetFreq ( 0, DDSfreq[0], DDSphase[0] ); // reset to F0/P0
            SetChannel (OUT_F0|OUT_P0);
            DDSmode = 0;
            break; // Mode: case 4
         } // DDSmode
         break;

      case 6:	// amplitude
			sprintf ( LCDstring, "Amp= %03d", Amplitude );
			Amplitude = (int)LCDvalue(3); // get and save new value
         SetAmplitude();
      	break;
      } // MenuItemNbr
	} // end of forever loop
}

void DDSramp ( void )
{
	LCDclear ();
	LCDprint ( "Ramp Step Vals" );
	sprintf ( LCDstring, "Size=%07ld", StepSize );
	StepSize = LCDvalue(7);					// get and save step size
	sprintf ( LCDstring, "Dur= %05d   ", StepDuration ); // extra spaces to clear size value
	StepDuration = (int)LCDvalue(5);		// get and save step delay
}

int SetRampFreq ( void )
{
	FSetFreq ( AdjCurFreq );
	if ( StepDuration ) MsDelay(StepDuration);
   if ( ReadSws & SW2 )
   {	LCDclear();								// clear display and set to line 1
	   sprintf ( LCDstring, "Ramp + stop at" );
		if ( !(SweepCount&1) ) LCDstring[5] = '-';
		LCDprint ( LCDstring );
	   LCDline ( 2 );
   	sprintf ( LCDstring, "%07ld Hz", (long)(AdjCurFreq/FREQ_FACTOR+0.5) );
		LCDprint ( LCDstring );
		while ( Switches() & SW2 );		// wait for SW2 release
		while ( !(Switches() & SW2) );	// wait for SW2 press
		while ( Switches() & SW2 );		// wait for SW2 release
		ShowDDS ();								// show DDS selections
   }
	return ( ExitFlag = ReadSws & SW1 );
}

void TestPSK ( void )
{
	BitWrPortI ( PFDDR, &PFDDRShadow, 1,6 ); // PF6 = output
	while (1)
	{	BitWrPortI ( PFDR, &PFDRShadow, 1, 6 );
		SetChannel (OUT_F0|OUT_P0);
		BitWrPortI ( PFDR, &PFDRShadow, 0 ,6 );
		SetChannel (OUT_F0|OUT_P1);
   }
}

